--[[
	KillingSpree2 authored by Douglas Craig (chexsum) <chexsum@gmail.com>

	Announces consecutive killing blows. 

]]
KillingSpree = LibStub("AceAddon-3.0"):NewAddon("KillingSpree", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0", "LibSink-2.0")
local self = KillingSpree

local MAXVICTIMS = 25   -- Maximum overall victims to show
local MULTITIMER = 120   -- Seconds given to perform multi kills
local SPREETIMER = 600  -- Seconds given to perform spree kills

local string_format = string.format
local time = time
local UnitSex = UnitSex
local UnitGUID = UnitGUID
local UnitName = UnitName
local UnitLevel = UnitLevel
local UnitExists = UnitExists
local UnitIsPlayer = UnitIsPlayer
local UnitIsVisible = UnitIsVisible
local UnitCanAttack = UnitCanAttack
local PlaySoundFile = PlaySoundFile
local GameTooltip = GameTooltip
local UIFrameFlash = UIFrameFlash
local LowHealthFrame = LowHealthFrame
local GetRealZoneText = GetRealZoneText
local PlaySoundFile = PlaySoundFile
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local CombatLog_Object_IsA = CombatLog_Object_IsA
local COMBATLOG_FILTER_MINE = COMBATLOG_FILTER_MINE
local COMBATLOG_FILTER_NEUTRAL_UNITS = COMBATLOG_FILTER_NEUTRAL_UNITS
local COMBATLOG_FILTER_HOSTILE_UNITS = COMBATLOG_FILTER_HOSTILE_UNITS
local COMBATLOG_FILTER_HOSTILE_PLAYERS = COMBATLOG_FILTER_HOSTILE_PLAYERS

local STRING_COLOR_BLACK = string_format("|cff%02x%02x%02x", 0, 0, 0)
local STRING_COLOR_WHITE = string_format("|cff%02x%02x%02x", 255, 255, 255)
local STRING_COLOR_YELLOW= string_format("|cff%02x%02x%02x", 255, 255, 0)
local STRING_COLOR_ORANGE = string_format("|cff%02x%02x%02x", 255, 128, 0)
local STRING_COLOR_CRIMSON = string_format("|cff%02x%02x%02x", 255, 0, 0)
local STRING_COLOR_MAROON = string_format("|cff%02x%02x%02x", 128, 0, 0)
local STRING_COLOR_GREEN = string_format("|cff%02x%02x%02x", 0, 255, 0)
local STRING_COLOR_BLUE = string_format("|cff%02x%02x%02x", 0, 0, 255)

function KillingSpree:OnInitialize()
	self.scheme = {}
	self.schemes = { "UT2k3", "UT2k4Male", "UT2k4Male2", "UT2k4Female", "UT2k4Female2" }
	for scheme = 1, #self.schemes do
		self.scheme[self.schemes[scheme]] = {
			['headshot'] = { "headshot.wav", "Headshot!" },
			['firstblood'] = { "firstblood.wav", "First Blood!" },
			['youhavewon'] = { "youhavewon.wav", "You have won the match!" },
			['youhavelost'] = { "youhavelost.wav", "You have lost the match!" },
			['multi'] = {
				[2] = { "doublekill.wav", "Double Kill!" },
				[3] = { "multikill.wav", "Multi Kill!" },
				[4] = { "megakill.wav", "Mega Kill!" },
				[5] = { "ultrakill.wav", "Ultra Kill!" },
				[6] = { "monsterkill.wav", "Monster Kill!" },
			},
			['spree'] = {
				[10] = { "killingspree.wav", "Killing Spree!" },
				[13] = { "rampage.wav", "Rampage!" },
				[16] = { "dominating.wav", "Dominating!" },
				[19] = { "unstoppable.wav", "Unstoppable!" },
				[22] = { "godlike.wav", "God-like!" },
			},
			['count'] = {
				[1] = { "1.wav", "One!" },
				[2] = { "2.wav", "Two!" },
				[3] = { "3.wav", "Three!" },
				[4] = { "4.wav", "Four!" },
				[5] = { "5.wav", "Five!" },
			},
		}
		if self.schemes[scheme] ~= "UT2k3" then
			self.scheme[self.schemes[scheme]]['humiliation'] = { "humiliation.wav", "Humiliating defeat!" }
			self.scheme[self.schemes[scheme]]['flawless'] = { "flawless.wav", "Flawless victory!" }
			self.scheme[self.schemes[scheme]]['multi'][7] = { "ludicrouskill.wav", "Ludicrous Kill!" }
			self.scheme[self.schemes[scheme]]['spree'][25] = { "wickedsick.wav", "Wicked Sick!" }
		end
	end
	if UnitSex('player') == 3 then
		self.defaultscheme = "UT2k4Female"
	else
		self.defaultscheme = "UT2k4Male"
	end
	self.defaults = {
		profile = {
			pvp = true,
			pve = false,
			scheme = self.defaultscheme,
		},
	}
	self.db = LibStub("AceDB-3.0"):New("KillingSpreeDB", self.defaults, "Default")
	self.options = {
		type = 'group',
		name = "KillingSpree",
		desc = "Consecutive kill announcer",
		icon = "Interface\\AddOns\\KillingSpree\\icon",
		args = {
			pvp = {
				order = 1,
				type = 'toggle',
				name = "Show PVP",
				desc = "Show PVP kills",
				get = function(info)
					return self.db.profile.pvp
				end,
				set = function(info, value)
					self.db.profile.pvp = value
				end,
			},
			pve = {
				order = 2,
				type = 'toggle',
				name = "Show PVE",
				desc = "Show PVE kills",
				get = function(info)
					return self.db.profile.pve
				end,
				set = function(info, value)
					self.db.profile.pve = value
				end,
			},
			scheme = {
				order = 3,
				type = 'select',
				name = "Sound scheme",
				desc = "Selects the sound scheme",
				get = function(info)
					return self.db.profile.scheme
				end,
				set = function(info, value)
				    self.db.profile.scheme = value
				end,
				values = {
					["UT2k3"] = "UT2k3",
					["UT2k4Male"] = "UT2k4Male",
					["UT2k4Male2"] = "UT2k4Male2",
					["UT2k4Female"] = "UT2k4Female",
					["UT2k4Female2"] = "UT2k4Female2",
	            },
			},
		},
	}
	self.options.args.output = self:GetSinkAce3OptionsDataTable()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("KillingSpree", self.options, {"killingspree"})
	self.optionsDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrame = self.optionsDialog:AddToBlizOptions("KillingSpree", "KillingSpree")
	self:SetSinkStorage(self.db.profile)
	self.databroker = LibStub:GetLibrary("LibDataBroker-1.1")
	self:ResetVictims()
	self:UpdateData()
end

function KillingSpree:OnEnable()
	self:ResetVictims()
	self:RegisterEvent("PLAYER_DEAD", "ResetVictims")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ResetVictims")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
end

function KillingSpree:ResetVictims()
	self.kills = 0
	self.multi = 0
	self.spree = 0
	self.overall = 0
	self.victims = {}
end

function KillingSpree:CombatLogEvent(event, timestamp, eventType, sourceGUID, sourceName, sourceFlags, destinationGUID, destinationName, destinationFlags)
	if eventType and eventType == "PARTY_KILL" then
		if not CombatLog_Object_IsA(sourceFlags, COMBATLOG_FILTER_MINE) then return end
		if not self.db.profile.pve and CombatLog_Object_IsA(destinationFlags, COMBATLOG_FILTER_HOSTILE_UNITS) then return end
		if not self.db.profile.pvp and CombatLog_Object_IsA(destinationFlags, COMBATLOG_FILTER_HOSTILE_PLAYERS) then return end
		self.kills = self.kills + 1
		self.victims[self.kills] = {}
		self.victims[self.kills].tod = time()
		self.victims[self.kills].name = destinationName
		self.victims[self.kills].location = GetRealZoneText()
		self:UpdateVictims()
		self:Announce()
	end
end

function KillingSpree:UpdateVictims()
	local multi, spree, overall, refresh = 0, 0, 0
	for victim = self.kills, 1, -1 do
		if self.victims[victim] then
			local difference = time() - self.victims[victim].tod
			if difference < MULTITIMER then
				if not refresh or refresh > MULTITIMER - difference then
					refresh = MULTITIMER - difference
				end
				multi = multi + 1
			elseif difference < SPREETIMER then
				if not refresh or refresh > SPREETIMER - difference then
					refresh = SPREETIMER - difference
				end
				spree = spree + 1
			elseif multi + spree + overall < MAXVICTIMS then
				overall = overall + 1
			end
		end
	end
	self.overall = overall + spree + multi
	self.spree = spree + multi
	self.multi = multi
	self:UpdateData()
	if refresh then
	        if self.refresh then self:CancelTimer(self.refresh) end
		self.refresh = self:ScheduleTimer("UpdateVictims", refresh)
	end
end

function KillingSpree:Announce()
	if self.overall > 1 and self.scheme[self.db.profile.scheme]['spree'][self.spree] then
		self:Pour(string_format("%s", self.scheme[self.db.profile.scheme]['spree'][self.spree][2]), 1, 0, 0)
		PlaySoundFile(string_format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.db.profile.scheme, self.scheme[self.db.profile.scheme]['spree'][self.spree][1]))
		UIFrameFlash(LowHealthFrame, 0.25, 0.25, 0.5, false, 0, 0)
	elseif self.overall > 1 and self.scheme[self.db.profile.scheme]['multi'][self.multi] then
		self:Pour(string_format("%s", self.scheme[self.db.profile.scheme]['multi'][self.multi][2]), 1, 0, 0)
		PlaySoundFile(string_format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.db.profile.scheme, self.scheme[self.db.profile.scheme]['multi'][self.multi][1]))
		UIFrameFlash(LowHealthFrame, 0.25, 0.25, 0.5, false, 0, 0)
	elseif self.overall == 1 and self.scheme[self.db.profile.scheme]['firstblood'] then
		self:Pour(string_format("%s", self.scheme[self.db.profile.scheme]['firstblood'][2]), 1, 0, 0)
		PlaySoundFile(string_format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.db.profile.scheme, self.scheme[self.db.profile.scheme]['firstblood'][1]))
		UIFrameFlash(LowHealthFrame, 0.25, 0.25, 0.5, false, 0, 0)
	elseif self.scheme[self.db.profile.scheme]['headshot'] then
		self:Pour(string_format("%s", self.scheme[self.db.profile.scheme]['headshot'][2]), 1, 0, 0)
		PlaySoundFile(string_format("Interface\\AddOns\\KillingSpree\\Snds\\%s\\%s", self.db.profile.scheme, self.scheme[self.db.profile.scheme]['headshot'][1]))
		UIFrameFlash(LowHealthFrame, 0.25, 0.25, 0.5, false, 0, 0)
	end
end

-- Here be dragons!
function KillingSpree:UpdateData()
	if not self.databroker then return end
	if not self.launcher then
		self.launcher = self.databroker:NewDataObject("KillingSpreeLDB", {
			type = "data source",
			label = STRING_COLOR_WHITE .. " KillingSpree ",
			icon = "Interface\\Addons\\KillingSpree\\icon",
		})
	else
		if self.multi + self.spree > 0 then
			self.launcher.text = STRING_COLOR_ORANGE .. self.multi .. STRING_COLOR_CRIMSON .. ":" .. STRING_COLOR_YELLOW .. self.spree
		else
			self.launcher.text = ""
		end
		for victim = self.kills or 0, 1, -1 do
			if self.victims[victim] then
				local difference = time() - self.victims[victim].tod
				if difference < MULTITIMER then
					GameTooltip:AddLine(string_format("     %s     ", self.victims[victim].name), 1, 0.5, 0)
				elseif difference < SPREETIMER then
					GameTooltip:AddLine(string_format("     %s     ", self.victims[victim].name), 1, 1, 0)
				end
			end
		end
		self.launcher.OnEnter = function (self)
			GameTooltip:SetOwner(self, "ANCHOR_BOTTOM")
			local self = KillingSpree
			if self.multi + self.spree > 0 then
				GameTooltip:AddLine("   Victims:  ", 1, 0, 0)
				for victim = KillingSpree.kills, 1, -1 do
					if KillingSpree.victims[victim] then
						local difference = time() - KillingSpree.victims[victim].tod
						if difference < MULTITIMER then
							GameTooltip:AddLine(string_format("     %s     ", KillingSpree.victims[victim].name), 1, 0.5, 0)
						elseif difference < SPREETIMER then
							GameTooltip:AddLine(string_format("     %s     ", KillingSpree.victims[victim].name), 1, 1, 0)
						end
					end
				end
			else
				GameTooltip:ClearLines()                
			end
			GameTooltip:Show()
		end
		self.launcher.OnLeave = function()
			GameTooltip:Hide()
		end
		self.launcher.OnClick = function(frame, button)
			if button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
			end
		end
	end
end

